@@ -52,7 +52,7 @@ import org.mozilla.jss.provider.java.security.JSSMessageDigestSpi;
  * Initialization is done with static methods, and must be done before
  * an instance can be created.  All other operations are done with instance
  * methods.
- * @version $Revision: 1.10 $ $Date: 2002/07/19 00:20:21 $
+ * @version $Revision: 1.15 $ $Date: 2002/12/13 06:12:28 $
  */
 public final class CryptoManager implements TokenSupplier
 {
@@ -62,19 +62,28 @@ public final class CryptoManager implements TokenSupplier
     public final static class CertUsage {
         private int usage;
         private String name;
+        static private ArrayList list = new ArrayList();
         private CertUsage() {};
         private CertUsage(int usage, String name) {
             this.usage = usage;
             this.name =  name;
+            this.list.add(this);
+
         }
-        int getUsage() {
+        public int getUsage() {
             return usage;
         }
 
+        static public Iterator getCertUsages() {
+            return list.iterator();
+
+        }
         public String toString() {
             return name;
         }
 
+
+
         // certUsage, these must be kept in sync with nss/lib/certdb/certt.h
         public static final CertUsage SSLClient = new CertUsage(0, "SSLClient");
         public static final CertUsage SSLServer = new CertUsage(1, "SSLServer");
@@ -731,6 +740,7 @@ public final class CryptoManager implements TokenSupplier
      */
     public synchronized native boolean FIPSEnabled();
 
+
     ///////////////////////////////////////////////////////////////////////
     // Password Callback management
     ///////////////////////////////////////////////////////////////////////
@@ -845,9 +855,6 @@ public final class CryptoManager implements TokenSupplier
                 instance.reloadModules();
             }
         }
-        if( values.removeSunProvider ) {
-            java.security.Security.removeProvider("SUN");
-        }
         if( values.installJSSProvider ) {
             // Force class load before we install the provider. Otherwise we get
             // an infinite loop as the Security manager tries to instantiate the
@@ -861,6 +868,9 @@ public final class CryptoManager implements TokenSupplier
                     "Unable to install default provider");
             }
         }
+        if( values.removeSunProvider ) {
+            java.security.Security.removeProvider("SUN");
+        }
     }
 
     private static native void
@@ -1292,38 +1302,26 @@ public final class CryptoManager implements TokenSupplier
     /********************************************************************/
 
     public static final String
-    JAR_JSS_VERSION     = "JSS_VERSION = JSS_3_2";
+    JAR_JSS_VERSION     = "JSS_VERSION = JSS_3_3";
     public static final String
     JAR_JDK_VERSION     = "JDK_VERSION = N/A";
     public static final String
-    JAR_NSS_VERSION     = "NSS_VERSION = NSS_3_4_2_RTM";
+    JAR_NSS_VERSION     = "NSS_VERSION = NSS_3_6_1_RTM";
     public static final String
     JAR_DBM_VERSION     = "DBM_VERSION = N/A";
     public static final String
-    JAR_NSPR_VERSION    = "NSPR_VERSION = NSPRPUB_RELEASE_4_2";
+    JAR_NSPR_VERSION    = "NSPR_VERSION = NSPRPUB_RELEASE_4_2_2";
 
     /**
      * Loads the JSS dynamic library if necessary.
-     * The system property "jss.load" will be set to "no" by jssjava
-     * because it is statically linked to the jss libraries. If this
-     * property is not set, that means we are not running jssjava
-     * and need to dynamically load the library.
      * <p>This method is idempotent.
      */
     synchronized static void loadNativeLibraries()
     {
-        if( ! mNativeLibrariesLoaded &&
-            ! ("no").equals(System.getProperty("jss.load")) )
+        if( ! mNativeLibrariesLoaded )
         {
-            try {
-                Debug.trace(Debug.VERBOSE, "about to load jss library");
-                System.loadLibrary("jss3");
-                Debug.trace(Debug.VERBOSE, "jss library loaded");
-            } catch( UnsatisfiedLinkError e) {
-                Debug.trace(Debug.ERROR, "ERROR: Unable to load jss library");
-                throw new UnsatisfiedLinkError(
-                    "Unable to load jss library or one of its dependencies");
-            }
+            System.loadLibrary("jss3");
+            Debug.trace(Debug.VERBOSE, "jss library loaded");
             mNativeLibrariesLoaded = true;
         }
     }
@@ -1433,4 +1431,36 @@ public final class CryptoManager implements TokenSupplier
         boolean checkSig, int cUsage)
         throws TokenException, CertificateEncodingException;
 
+     ///////////////////////////////////////////////////////////////////////
+    // OCSP management
+    ///////////////////////////////////////////////////////////////////////
+
+    /**
+     * Enables OCSP, note when you Initialize JSS for the first time, for
+     * backwards compatibility, the initialize will enable OCSP if you
+     * previously set values.ocspCheckingEnabled and
+     * values.ocspResponderURL/values.ocspResponderCertNickname
+     * configureOCSP will allow changing of the the OCSPResponder at runtime.
+     *      * @param ocspChecking true or false to enable/disable OCSP
+     *      * @param ocspResponderURL - url of the OCSP responder
+     *      * @param ocspResponderCertNickname - the nickname of the OCSP
+     *        signer certificate or the CA certificate found in the cert DB
+     */
+
+    public void configureOCSP(
+        boolean ocspCheckingEnabled,
+        String ocspResponderURL,
+        String ocspResponderCertNickname )
+    throws GeneralSecurityException
+    {
+        configureOCSPNative(ocspCheckingEnabled,
+                                   ocspResponderURL,
+                                    ocspResponderCertNickname );
+    }
+
+    private native void configureOCSPNative( boolean ocspCheckingEnabled,
+                    String ocspResponderURL,
+                    String ocspResponderCertNickname )
+                    throws GeneralSecurityException;
+
 }
