@@ -260,7 +260,7 @@ Java_org_mozilla_jss_CryptoManager_findPrivKeyByCertNative
     PRThread *pThread;
     CERTCertificate *cert;
     SECKEYPrivateKey *privKey=NULL;
-    jobject Key;
+    jobject Key = NULL;
 
     pThread = PR_AttachThread(PR_SYSTEM_THREAD, 0, NULL);
     PR_ASSERT( pThread != NULL);
@@ -973,11 +973,8 @@ Java_org_mozilla_jss_CryptoManager_importCertPackageNative
                                     numCerts-userCertFound,
                                     certUsageUserCertImport);
         if(status != SECSuccess) {
-            JSS_trace(env, JSS_TRACE_ERROR,
-                "CERT_ImportCAChain returned an error in "
-                "CryptoManager.importCertPackage.");
-            JSS_throwMsg(env, CERTIFICATE_ENCODING_EXCEPTION,
-                "CERT_ImportCAChain returned an error");
+            JSS_throwMsgPrErr(env, CERTIFICATE_ENCODING_EXCEPTION,
+                "CERT_ImportCAChainTrusted returned an error");
             goto finish;
         }
       } else if (certi == numCerts) {
@@ -985,11 +982,8 @@ Java_org_mozilla_jss_CryptoManager_importCertPackageNative
                                     numCerts-userCertFound,
                                     certUsageUserCertImport);
         if(status != SECSuccess) {
-            JSS_trace(env, JSS_TRACE_ERROR,
-                "CERT_ImportCAChain returned an error in "
-                "CryptoManager.importCertPackage.");
-            JSS_throwMsg(env, CERTIFICATE_ENCODING_EXCEPTION,
-                "CERT_ImportCAChain returned an error");
+            JSS_throwMsgPrErr(env, CERTIFICATE_ENCODING_EXCEPTION,
+                "CERT_ImportCAChainTrusted returned an error");
             goto finish;
         }
       } else {
@@ -997,11 +991,8 @@ Java_org_mozilla_jss_CryptoManager_importCertPackageNative
                    certi,
                    certUsageUserCertImport);
         if(status != SECSuccess) {
-            JSS_trace(env, JSS_TRACE_ERROR,
-                "CERT_ImportCAChain returned an error in "
-                "CryptoManager.importCertPackage.");
-            JSS_throwMsg(env, CERTIFICATE_ENCODING_EXCEPTION,
-                "CERT_ImportCAChain returned an error");
+            JSS_throwMsgPrErr(env, CERTIFICATE_ENCODING_EXCEPTION,
+                "CERT_ImportCAChainTrusted returned an error");
             goto finish;
         }
 
@@ -1009,11 +1000,8 @@ Java_org_mozilla_jss_CryptoManager_importCertPackageNative
                    numCerts-certi-1,
                    certUsageUserCertImport);
         if(status != SECSuccess) {
-            JSS_trace(env, JSS_TRACE_ERROR,
-                "CERT_ImportCAChain returned an error in "
-                "CryptoManager.importCertPackage.");
-            JSS_throwMsg(env, CERTIFICATE_ENCODING_EXCEPTION,
-                "CERT_ImportCAChain returned an error");
+            JSS_throwMsgPrErr(env, CERTIFICATE_ENCODING_EXCEPTION,
+                "CERT_ImportCAChainTrusted returned an error");
             goto finish;
         }
 
@@ -1036,7 +1024,11 @@ Java_org_mozilla_jss_CryptoManager_importCertPackageNative
     } else {
         leafCert = CERT_FindCertByIssuerAndSN(certdb, &issuerAndSN);
     }
-    PR_ASSERT( leafCert != NULL );
+    if( leafCert == NULL ) {
+        JSS_throwMsgPrErr(env, TOKEN_EXCEPTION,
+            "Failed to find certificate that was just imported");
+        goto finish;
+    }
     leafObject = JSS_PK11_wrapCert(env, &leafCert);
 
 finish:
