@@ -47,10 +47,18 @@ public interface SymmetricKey {
     public CryptoToken getOwningToken();
 
     public int getStrength();
+    public int getLength();
 
     public byte[] getKeyData() throws NotExtractableException;
 
-    public static class NotExtractableException extends Exception { }
+    public static class NotExtractableException extends Exception {
+        public NotExtractableException() {
+            super();
+        }
+        public NotExtractableException(String mesg) {
+            super(mesg);
+        }
+    }
 
     String getAlgorithm();
 
