@@ -232,6 +232,7 @@ Java_org_mozilla_jss_pkcs11_PK11Signature_engineSignNative
 
     PR_ASSERT(env!=NULL && this!=NULL);
 
+    signature.data = NULL;
 
     /*
      * Extract the signature context from the Java wrapper
@@ -246,7 +247,8 @@ Java_org_mozilla_jss_pkcs11_PK11Signature_engineSignNative
      * Finish the signing operation.
      */
     if( SGN_End(ctxt, &signature) != SECSuccess) {
-        JSS_throw(env, SIGNATURE_EXCEPTION);
+        JSS_throwMsgPrErr(env, SIGNATURE_EXCEPTION,
+            "Signing operation failed");
         goto finish;
     }
 
@@ -264,13 +266,14 @@ Java_org_mozilla_jss_pkcs11_PK11Signature_engineSignNative
         goto finish;
     }
     memcpy(sigBytes, signature.data, signature.len);
-    (*env)->ReleaseByteArrayElements(env, sigArray, sigBytes, 0);
-    sigBytes=NULL;
     
 finish:
     if(sigBytes != NULL) {
         (*env)->ReleaseByteArrayElements(env, sigArray, sigBytes, 0);
     }
+    if( signature.data != NULL ) {
+        PR_Free(signature.data);
+    }
     return sigArray;
 }
 
